package java.lang.reflect;

import java.security.AccessController;
import sun.reflect.Reflection;
import java.lang.annotation.Annotation;
import sun.reflect.ReflectionFactory;
import java.security.Permission;

public class AccessibleObject implements AnnotatedElement
{
    private static final Permission ACCESS_PERMISSION;
    boolean override;
    static final ReflectionFactory reflectionFactory;
    volatile Object securityCheckCache;
    
    public static void setAccessible(AccessibleObject[] array, boolean b) throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(AccessibleObject.ACCESS_PERMISSION);
        }
        for (int i = 0; i < array.length; ++i) {
            setAccessible0(array[i], b);
        }
    }
    
    public void setAccessible(boolean b) throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(AccessibleObject.ACCESS_PERMISSION);
        }
        setAccessible0(this, b);
    }
    
    private static void setAccessible0(AccessibleObject accessibleObject, boolean override) throws SecurityException {
        if (accessibleObject instanceof Constructor && override && ((Constructor)accessibleObject).getDeclaringClass() == Class.class) {
            throw new SecurityException("Cannot make a java.lang.Class constructor accessible");
        }
        accessibleObject.override = override;
        return;
    }
    
    public boolean isAccessible() {
        return this.override;
    }
    
    protected AccessibleObject() {
    }
    
    @Override
    public <T extends Annotation> T getAnnotation(Class<T> clazz) {
        throw new AssertionError("All subclasses should override this method");
    }
    
    @Override
    public boolean isAnnotationPresent(Class<? extends Annotation> clazz) {
        return super.isAnnotationPresent(clazz);
    }
    
    @Override
    public <T extends Annotation> T[] getAnnotationsByType(Class<T> clazz) {
        throw new AssertionError("All subclasses should override this method");
    }
    
    @Override
    public Annotation[] getAnnotations() {
        return this.getDeclaredAnnotations();
    }
    
    @Override
    public <T extends Annotation> T getDeclaredAnnotation(Class<T> clazz) {
        return this.getAnnotation(clazz);
    }
    
    @Override
    public <T extends Annotation> T[] getDeclaredAnnotationsByType(Class<T> clazz) {
        return this.getAnnotationsByType(clazz);
    }
    
    @Override
    public Annotation[] getDeclaredAnnotations() {
        throw new AssertionError("All subclasses should override this method");
    }
    
    void checkAccess(Class<?> clazz, Class<?> class1, Object o, int n) throws IllegalAccessException {
        if (clazz == class1) {
            return;
        }
        Object securityCheckCache = this.securityCheckCache;
        if (o != null && Modifier.isProtected(n) && (class1 = o.getClass()) != class1) {
            if ((securityCheckCache instanceof Class[])) {
                Class[] array = (Class[])(Class[])securityCheckCache;
                if (array[1] == class1 && array[0] == clazz) {
                    return;
                }
            }
        }
        else if (securityCheckCache == clazz) {
            return;
        }
        this.slowCheckMemberAccess(clazz, class1, o, n, class1);
        return;
    }
    
    void slowCheckMemberAccess(Class<?> clazz, Class<?> clazz2, Object o, int n, Class<?> clazz3) throws IllegalAccessException {
        Reflection.ensureMemberAccess(clazz, clazz2, o, n);
        this.securityCheckCache = ((clazz3 == clazz2) ? clazz : new Class[] { clazz, clazz3 });
    }
    
    static {
        ACCESS_PERMISSION = new ReflectPermission("suppressAccessChecks");
        reflectionFactory = (ReflectionFactory)AccessController.doPrivileged(new ReflectionFactory.GetReflectionFactoryAction());
    }
}
